@@ -19,7 +19,16 @@ CREATE TYPE state_type AS ENUM ( 'hard', 'soft' );
 CREATE TYPE checkable_type AS ENUM ( 'host', 'service' );
 CREATE TYPE comment_type AS ENUM ( 'comment', 'ack' );
 CREATE TYPE notification_type AS ENUM ( 'downtime_start', 'downtime_end', 'downtime_removed', 'custom', 'acknowledgement', 'problem', 'recovery', 'flapping_start', 'flapping_end' );
-CREATE TYPE history_type AS ENUM ( 'notification', 'state_change', 'downtime_start', 'downtime_end', 'comment_add', 'comment_remove', 'flapping_start', 'flapping_end', 'ack_set', 'ack_clear' );
+
+-- The enum values are ordered in a way that event_type provides a meaningful sort order for history entries with
+-- the same event_time. state_change comes first as it can cause many of the other events like trigger downtimes,
+-- remove acknowledgements and send notifications. Similarly, notification comes last as any other event can result
+-- in a notification. End events sort before the corresponding start events as any ack/comment/downtime/flapping
+-- period should last for more than a millisecond, therefore, the old period ends first and then the new one starts.
+-- The remaining types are sorted by impact and cause: comments are informative, flapping is automatic and changes
+-- mechanics, downtimes are semi-automatic, require user action (or configuration) and change mechanics, acks are pure
+-- user actions and change mechanics.
+CREATE TYPE history_type AS ENUM ( 'state_change', 'ack_clear', 'downtime_end', 'flapping_end', 'comment_remove', 'comment_add', 'flapping_start', 'downtime_start', 'ack_set', 'notification' );
 
 CREATE OR REPLACE FUNCTION get_sla_ok_percent(
   in_host_id bytea20,
@@ -305,6 +314,8 @@ ALTER TABLE hostgroup ALTER COLUMN name_checksum SET STORAGE PLAIN;
 ALTER TABLE hostgroup ALTER COLUMN properties_checksum SET STORAGE PLAIN;
 ALTER TABLE hostgroup ALTER COLUMN zone_id SET STORAGE PLAIN;
 
+CREATE INDEX idx_hostgroup_display_name ON hostgroup(display_name);
+CREATE INDEX idx_hostgroup_name_ci ON hostgroup(name_ci);
 CREATE INDEX idx_hostgroup_name ON hostgroup(name);
 
 COMMENT ON COLUMN hostgroup.id IS 'sha1(environment.id + name)';
@@ -313,7 +324,9 @@ COMMENT ON COLUMN hostgroup.name_checksum IS 'sha1(name)';
 COMMENT ON COLUMN hostgroup.properties_checksum IS 'sha1(all properties)';
 COMMENT ON COLUMN hostgroup.zone_id IS 'zone.id';
 
-COMMENT ON INDEX idx_hostgroup_name IS 'Host/service/host group list filtered by host group name';
+COMMENT ON INDEX idx_hostgroup_display_name IS 'Hostgroup list filtered/ordered by display_name';
+COMMENT ON INDEX idx_hostgroup_name_ci IS 'Hostgroup list filtered using quick search';
+COMMENT ON INDEX idx_hostgroup_name IS 'Host/service/host group list filtered by host group name; Hostgroup detail filter';
 
 CREATE TABLE hostgroup_member (
   id bytea20 NOT NULL,
@@ -577,8 +590,12 @@ COMMENT ON COLUMN servicegroup.name_checksum IS 'sha1(name)';
 COMMENT ON COLUMN servicegroup.properties_checksum IS 'sha1(all properties)';
 COMMENT ON COLUMN servicegroup.zone_id IS 'zone.id';
 
+CREATE INDEX idx_servicegroup_display_name ON servicegroup(display_name);
+CREATE INDEX idx_servicegroup_name_ci ON servicegroup(name_ci);
 CREATE INDEX idx_servicegroup_name ON servicegroup(name);
-COMMENT ON INDEX idx_servicegroup_name IS 'Host/service/service group list filtered by service group name';
+COMMENT ON INDEX idx_servicegroup_display_name IS 'Servicegroup list filtered/ordered by display_name';
+COMMENT ON INDEX idx_servicegroup_name_ci IS 'Servicegroup list filtered using quick search';
+COMMENT ON INDEX idx_servicegroup_name IS 'Host/service/service group list filtered by service group name; Servicegroup detail filter';
 
 CREATE TABLE servicegroup_member (
   id bytea20 NOT NULL,
@@ -1235,7 +1252,7 @@ CREATE TABLE notification (
   name_checksum bytea20 NOT NULL,
   properties_checksum bytea20 NOT NULL,
 
-  name varchar(255) NOT NULL,
+  name varchar(767) NOT NULL,
   name_ci citext NOT NULL,
 
   host_id bytea20 NOT NULL,
@@ -1271,6 +1288,7 @@ CREATE INDEX idx_notification_service_id ON notification(service_id);
 COMMENT ON COLUMN notification.id IS 'sha1(environment.id + name)';
 COMMENT ON COLUMN notification.environment_id IS 'environment.id';
 COMMENT ON COLUMN notification.name_checksum IS 'sha1(name)';
+COMMENT ON COLUMN notification.name IS '255+1+255+1+255, i.e. "host.name!service.name!notification.name"';
 COMMENT ON COLUMN notification.host_id IS 'host.id';
 COMMENT ON COLUMN notification.service_id IS 'service.id';
 COMMENT ON COLUMN notification.notificationcommand_id IS 'command.id';
@@ -1552,7 +1570,7 @@ CREATE TABLE customvar_flat (
   flatname_checksum bytea20 NOT NULL,
 
   flatname citext NOT NULL,
-  flatvalue text NOT NULL,
+  flatvalue text DEFAULT NULL,
 
   CONSTRAINT pk_customvar_flat PRIMARY KEY (id)
 );
@@ -1563,6 +1581,7 @@ ALTER TABLE customvar_flat ALTER COLUMN customvar_id SET STORAGE PLAIN;
 ALTER TABLE customvar_flat ALTER COLUMN flatname_checksum SET STORAGE PLAIN;
 
 CREATE INDEX idx_customvar_flat_customvar_id ON customvar_flat(customvar_id);
+CREATE INDEX idx_customvar_flat_flatname_flatvalue ON customvar_flat(flatname, flatvalue);
 
 COMMENT ON COLUMN customvar_flat.id IS 'sha1(environment.id + flatname + flatvalue)';
 COMMENT ON COLUMN customvar_flat.environment_id IS 'environment.id';
@@ -1570,6 +1589,8 @@ COMMENT ON COLUMN customvar_flat.customvar_id IS 'sha1(customvar.id)';
 COMMENT ON COLUMN customvar_flat.flatname_checksum IS 'sha1(flatname after conversion)';
 COMMENT ON COLUMN customvar_flat.flatname IS 'Path converted with `.` and `[ ]`';
 
+COMMENT ON INDEX idx_customvar_flat_flatname_flatvalue IS 'Lists filtered by custom variable';
+
 CREATE TABLE "user" (
   id bytea20 NOT NULL,
   environment_id bytea20 NOT NULL,
@@ -2028,7 +2049,7 @@ CREATE TABLE history (
   flapping_history_id bytea20 DEFAULT NULL,
   acknowledgement_history_id bytea20 DEFAULT NULL,
 
-  event_type history_type NOT NULL DEFAULT 'notification',
+  event_type history_type NOT NULL DEFAULT 'state_change',
   event_time biguint NOT NULL,
 
   CONSTRAINT pk_history PRIMARY KEY (id),
@@ -2160,4 +2181,4 @@ CREATE TABLE icingadb_schema (
 ALTER SEQUENCE icingadb_schema_id_seq OWNED BY icingadb_schema.id;
 
 INSERT INTO icingadb_schema (version, timestamp)
-  VALUES (1, extract(epoch from now()) * 1000);
+  VALUES (2, extract(epoch from now()) * 1000);
